#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Creating an Explicit Structured Grid
====================================

This example shows how to create an explicit structured grid.

Author: Rodrigo Mologni G. dos Santos <rodrigo@solpe.com.br>
        SOLPE - Solutions for Petroleum Science and Engineering
        https://www.solpe.com.br/

"""

import vtk
import numpy as np

from vtk.util.numpy_support import numpy_to_vtk
from pyvista.utilities.helpers import vtk_points
from pyvista.utilities.cells import CellArray

# Loads pre-processed data from the UNISIM-II reservoir model
# https://www.unisim.cepetro.unicamp.br/benchmarks/br/unisim-ii/overview
ni, nj, nk = (46, 69, 15)
points = np.load('points.npy')
cells = np.load('cells.npy')
blank_cells = np.load('blank_cells.npy')

# Converts from NumPy to VTK using PyVista methods
points = vtk_points(points)
cells = CellArray(cells)

# Creates the ghost cell array
ghost_cells = np.zeros_like(blank_cells)
ind = np.argwhere(blank_cells == 0)
ghost_cells[ind] = vtk.vtkDataSetAttributes.HIDDENCELL
ghost_cells = numpy_to_vtk(ghost_cells)
name = vtk.vtkDataSetAttributes.GhostArrayName()
ghost_cells.SetName(name)

# See https://blog.kitware.com/ghost-and-blanking-visibility-changes/
grid = vtk.vtkExplicitStructuredGrid()
grid.SetDimensions(ni+1, nj+1, nk+1)
grid.SetPoints(points)
grid.SetCells(cells)  # shouldn't be required because all cells are hexahedrons
grid.GetCellData().AddArray(ghost_cells)
grid.GlobalWarningDisplayOff()  # hide errors caused by ghost cells

mapper = vtk.vtkDataSetMapper()
mapper.SetInputData(grid)

actor = vtk.vtkActor()
actor.SetMapper(mapper)
actor.GetProperty().EdgeVisibilityOn()
actor.GetProperty().LightingOff()

renderer = vtk.vtkRenderer()
renderer.AddActor(actor)
renderer.SetBackground(.3, .3, .3)

window = vtk.vtkRenderWindow()
window.AddRenderer(renderer)
window.SetSize(800, 600)

interactor = vtk.vtkRenderWindowInteractor()
interactor.SetRenderWindow(window)
interactor.SetInteractorStyle(vtk.vtkInteractorStyleRubberBandPick())
interactor.Start()
